import enviro, time, sys 
from math import ceil
from as7341 import *
from ads1115 import *
from tcs3472 import tcs3472
from machine import Pin, I2C
from picographics import PicoGraphics, DISPLAY_INKY_PACK

import enviro
import os
from enviro.constants import *

from pimoroni_i2c import PimoroniI2C

# Wyczyszczenie ekranu
def reset_screen():
    inky.set_pen(15)
    inky.clear()
    inky.update()

# Pomiary z AS7341
def get_light(as7341_sensor):
    fx = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    as7341_sensor.start_measure("F1F4CN")
    fx[0], fx[1], fx[2], fx[3], fx[4], fx[5] = as7341_sensor.get_spectral_data()
    
    as7341_sensor.start_measure("F5F8CN")
    fx[4], fx[5], fx[6], fx[7], fx[8], fx[9] = as7341_sensor.get_spectral_data()
    
    fx[10] = as7341_sensor.get_flicker_frequency()
    
    kolory = [
        "405-425nm",
        "435-455nm",
        "470-490nm",
        "505-525nm",
        "545-565nm",
        "580-600nm",
        "620-640nm",
        "670-690nm"
    ]
    
    return [[str(x) for x in fx], kolory[fx.index(max(fx[0:8]))]]

# Inicjalizacja i sprawdzenie polaczen
# BME280 0x77[119] 3,4MHz
# PCF85063A 0x51[81] 400kHz
# LTR-559ALS-01 0x23[35] 400kHz
# AS7341  0x39[57] 400kHz
# TCS34725 0x29[41] 400kHz
# ADS1115 0x48[72] 3,4MHz
def check_connections():
    #i2c1 = I2C(1, PIN=4, PIN-5, freq=100000)
    i2c1 = PimoroniI2C(I2C_SDA_PIN, I2C_SCL_PIN, 100000) # GP4, GP5
    i2c_dev_list = i2c1.scan()
    if len(i2c_dev_list) == 0:
        print("No I2C devices found, terminating")
        sys.exit(1)
    print("LTR-559ALS-01 0x23, PCF85063A 0x51, BME280 0x77, ADS1115 0x48/0x49, AS7341 0x39, TCS34725 0x29")
    #print("Detected devices at I2C1-addresses:",
    #    " ".join(["0x{:02X}".format(x) for x in i2c_dev_list]))
    print("Detected devices at I2C1-addresses:")
    print(["0x{:02X}".format(x) for x in i2c_dev_list])
    print(i2c_dev_list)

    # Sprawdzanie AS7341  0x39[57]
    if not 57 in i2c_dev_list:
    #if not as7341_sensor.isconnected():
        #print("Failed to contact AS7341, terminating")
        #sys.exit(1)
        print("Failed to contact AS7341")
        as_flag = False
    else:
        as_flag = True       
    
    # Sprawdzanie ADS1115  0x48[72]
    #if not 72 in I2C(0, sda=Pin(4), scl=Pin(5)).scan():
    if not 72 in i2c_dev_list:
        #print("Failed to contact ADS1115, terminating")
        #sys.exit(1)
        print("Failed to contact ADS1115")
        ads_flag = False
    else:
        ads1115_flag = True

    # Sprawdzanie TCS34725 Grove I2C  0x29[41]
    if not 41 in i2c_dev_list:
        print("Failed to contact TCS34725")
        tcs_flag = False
    else:
        tcs_flag = True       
        
    #time.sleep(1)
    return [as_flag, tcs_flag, ads1115_flag]

# Wyprowadzenie na ekran informacji z Enviro i UV
def build_info(enviro_readings, fx_reading, uv):
    inky.set_pen(15)
    inky.clear()
    inky.set_pen(0)
    inky.set_font("bitmap8")

    # Date and time
    now_str = enviro.helpers.datetime_string()
    inky.text(f"{now_str}", 0, 0, scale=1.5)

    #inky.text("Cisnienie", 0, 90, scale=2.5)
    inky.text(str(enviro_readings["pressure"]) + " hPa", 0, 10, scale=2.0)
    #inky.text("Temperatura", 0, 0, scale=2.5)
    inky.text(str(enviro_readings["temperature"]) + "°C", 0, 30, scale=2.0)
    #inky.text("Wilgotnosc", 0, 45, scale=2.5)
    inky.text(str(enviro_readings["humidity"]) + "%", 90, 30, scale=2.0)
    
    if ads1115_flag == True: # ADS1115   
        #inky.text("Indeks UV", 140, 90, scale=2.5)
        inky.text("UV:" + str(ceil(uv)) + "mV", 90, 70, scale=2.0)
        print("UV: " + str(uv))
    
    if as_flag == True: # AS7341
        #"luminance"  ENVIRO   AS clear
        inky.text("CAS:" + fx_reading[8] , 0, 90, scale=2.0)
        inky.text("ALS: " + str(enviro_readings["luminance"]) + " LUX", 0, 50, scale=2.0)
        # Flicker
        if (fx_reading[10] == '0'):
            inky.text("FL:-" , 0, 70, scale=2)
        else:
            inky.text("FL:" + fx_reading[10] + "Hz", 0, 70, scale=2)
        # Spectral
        inky.text("F1", 180, 0, scale=2.5)
        inky.text("F2", 220, 0, scale=2.5)
        inky.text("F3", 260, 0, scale=2.5)
        inky.text(fx_reading[0], 180, 20, scale=2.0)
        inky.text(fx_reading[1], 220, 20, scale=2.0)
        inky.text(fx_reading[2], 260, 20, scale=2.0)
    
        inky.text("F4", 180, 45, scale=2.5)
        inky.text("F5", 220, 45, scale=2.5)
        inky.text("F6", 260, 45, scale=2.5)
        inky.text(fx_reading[3], 180, 65, scale=2.0)
        inky.text(fx_reading[4], 220, 65, scale=2.0)
        inky.text(fx_reading[5], 260, 65, scale=2.0)
    
        inky.text("F7", 180, 90, scale=2.5)
        inky.text("F8", 220, 90, scale=2.5)
        inky.text("NIR", 260, 90, scale=2.5)
        inky.text(fx_reading[6], 180, 110, scale=2.0)
        inky.text(fx_reading[7], 220, 110, scale=2.0)
        inky.text(fx_reading[9], 260, 110, scale=2.0)

    if tcs_flag == True: # TCS34725
        # CRGB suma
        #inky.text("CTCS:" + grove_readings[3], 90, 90, scale=2)
        inky.text("CTCS:" + str(tcs.light()), 90, 90, scale=2)
        #inky.text("RGB:" + grove_readings[1] + " " + grove_readings[0] + " " + grove_readings[2] , 60, 95, scale=2) #OLD
        #inky.text("TCS RGB: " + grove_readings[1] + " " + grove_readings[0] + " " + grove_readings[2] , 0, 110, scale=2)
        inky.text("TCS RGB: " + str(tuple(tcs.rgb())[0]) + " " + str(tuple(tcs.rgb())[1]) + " " + str(tuple(tcs.rgb())[2]) , 0, 110, scale=2)
 
    inky.update()

#========================================
as_flag = False	# AS7341
as_previous_flag = False

tcs_flag = False	# TCS34725
tcs_pewvious_flag = False
ads1115_flag = False	# ADS1115
ads1115_previous_flag = False

# Inicjalizacja wyświetlaczsa i modułów
inky = PicoGraphics(display = DISPLAY_INKY_PACK)
reset_screen()

while True:
    [as_flag, tcs_flag, ads1115_flag] = check_connections()

    # Inicjalizacja modułów I2C
    if (as_flag == True) & (as_previous_flag == False):  # AS7341
        print("AS7341 initialization")
        #as7341_sensor = AS7341(I2C(0,scl=5, sda=4, freq=4000))
        as7341_sensor = AS7341(PimoroniI2C(I2C_SDA_PIN, I2C_SCL_PIN, 100000))    
        as7341_sensor.set_measure_mode(AS7341_MODE_SPM)
        as7341_sensor.set_atime(29)
        as7341_sensor.set_astep(599)
        as7341_sensor.set_again(4)
        as_previous_flag = True

    if (tcs_flag == True) & (tcs_pewvious_flag == False): # TCS34725
        print("TCS34725 initialization")
        #grove_i2c = I2C(0, scl=5, sda=4, freq=4000)
        #grove_i2c = PimoroniI2C(I2C_SDA_PIN, I2C_SCL_PIN, 100000)
        bus = PimoroniI2C(I2C_SDA_PIN, I2C_SCL_PIN, 100000)
        tcs = tcs3472(bus)
        tcs_pewvious_flag = True


    if (ads1115_flag == True) & (ads1115_previous_flag == False): # ADS1115
        print("ADS1115 initialization")
        adc = ADS1115(72, i2c=PimoroniI2C(I2C_SDA_PIN, I2C_SCL_PIN, 100000))
        adc.setVoltageRange_mV(ADS1115_RANGE_2048)	#ADS1115_RANGE_6144
        adc.setCompareChannels(ADS1115_COMP_0_GND)
        ads1115_previous_flag = True 

    # Szybki odczyt informacji
    enviro_readings = enviro.get_sensor_readings()
    
    if (tcs_flag == True): # TCS34725
        print("Light:", tcs.light())
        print("RGB:", tcs.rgb())
        #grove_readings = grove.get_data(grove_i2c)
        #print(grove_readings) # GRBC
    
    if as_flag == True:
        [fx_reading, color_reading] = get_light(as7341_sensor)
        
    if ads1115_flag == True:        
        uv = adc_readChannel(adc, ADS1115_COMP_0_GND) # HKW modyfied

    # Wyswietl
    build_info(enviro_readings, fx_reading, uv)
    
    # Uśpienie całego układu na minutę
    enviro.sleep(1)
#
